package	jp.co.sds_corp.util.msg;

import java.util.Hashtable;

import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* メッセージ処理アダプター
*/
public interface IMessageAdapter extends IConfigSavable
{
	/**
	* ＪＭＳに接続します
	* @param	env		接続パラメータ
	*/
	public void connect( Hashtable env ) throws Exception;
	/**
	* ＪＭＳに接続します
	* @param	config	接続パラメータ
	*/
	public void connect( IConfiguration config ) throws Exception;

	/**
	* 送信リソースを生成します
	*/
	public void createSender() throws Exception;
	/**
	* 送信リソースを生成します
	* @param	to		送信先
	*/
	public void createSender( String to ) throws Exception;
	/**
	* 受信リソースを生成します
	* @param	cond	受信条件
	* @param	wait	(true: wait false: non wait)
	*/
	public void createReceiver( String cond, boolean wait )
		throws Exception;
	/**
	* 受信リソースを生成します
	* @param	from	受信元
	* @param	cond	受信条件
	* @param	wait	(true: wait false: non wait)
	*/
	public void createReceiver( String from, String cond, boolean wait )
		throws Exception;

	/**
	* メッセージを送信します
	* @param	msg		送信するメッセージ
	* @param	expiration	メッセージの有効期限
	* @return	waitがtrueの場合は受信したメッセージを、それ以外はヌルを戻す
	*/
	public IMessage writeMessage( IMessage msg, long expiration )
		throws Exception;
	/**
	* メッセージを受信します
	* @return	受信したメッセージ
	*/
	public IMessage receiveMessage() throws Exception;

	/**
	* メッセージを生成します
	* @param	type	メッセージタイプ
	* @return	メッセージ
	*/
	public IMessage createMessage( IMessage.Type type ) throws Exception;
	/**
	* メッセージの複製を作成します
	* @param	msg		コピー元
	* @return	複製されたメッセージ
	*/
	public IMessage copyMessage( IMessage msg ) throws Exception;

	/**
	* メッセージングサービスを開始
	*/
	public void start() throws Exception;
	/**
	* メッセージングサービスを中断します
	*/
	public void stop() throws Exception;
	/**
	* メッセージングサービスを閉じます
	*/
	public void close() throws Exception;

	/**
	* メッセージの受信コールバックを設定します。
	* @param	listener	メッセージの受信コールバック
	*/
	public void addReceiveListener( IReceiveListener listener );
	/**
	* 例外の受信コールバックを登録します。
	* @param	listener	例外の受信コールバック
	*/
	public void addExceptionListener( IExceptionListener listener );

	/**
	* 受信タイムアウトを設定します。
	* @param	millisec	受信タイムアウト(ミリ秒)
	*/
	public void setReceiveTimeout( int millisec );
}
