package	jp.co.sds_corp.util.msg.jms;

import java.util.Hashtable;

import javax.jms.JMSException;
import javax.jms.Session;

import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.msg.MessageAdapterFactory;

import com.ibm.mq.MQC;
import com.ibm.mq.MQEnvironment;
import com.ibm.mq.jms.JMSC;
import com.ibm.mq.jms.MQQueueConnectionFactory;

/**
* JMS MQ-QUEUE メッセージングサービス
* IBM-MQとＪＮＤＩを使用しないで接続する仕組みです
*/
public class SimpleMqQueue extends JmsQueuePackage
{
	String qmgr;
	String host;
	String port;
	String channel;

	public final String MQ_QMGR	= "MQ_QMGR";
	public final String MQ_HOST	= "MQ_HOST";
	public final String MQ_PORT	= "MQ_PORT";
	public final String MQ_CHANNEL	= "MQ_CHANNEL";

	/**
	* MQに接続します。
	* @param	config	接続パラメータ
	*/
	public void connect( IConfiguration config ) throws Exception {
		qmgr = config.get( MQ_QMGR );
		host = config.get( MQ_HOST, null );
		port = config.get( MQ_PORT, null );
		channel = config.get( MQ_CHANNEL, null );
		user = config.get( MessageAdapterFactory.CONNECTION_USER, null );
		passwd = config.get( MessageAdapterFactory.CONNECTION_PASSWD, null );
		sendName = config.get( MessageAdapterFactory.DEST_SEND, null );
		recvName = config.get( MessageAdapterFactory.DEST_RECV, null );

		connect( qmgr, host, port, channel, user, passwd );
	}

	/**
	* MQに接続します。
	* @param	env		接続パラメータ
	*/
	public void connect( Hashtable env ) throws Exception {
		try {
			qmgr = (String )env.get( "MQ_QMGR" );
		} catch ( Exception ex ) {
		}
		host = null;
		port = null;
		channel = null;
		try {
			host = (String )env.get("MQ_HOST");
			port = (String )env.get("MQ_PORT");
			channel = (String )env.get("MQ_CHANNEL");
		} catch ( Exception ex ) {
		}
		user = (String)env.get( MessageAdapterFactory.CONNECTION_USER );
		passwd = (String)env.get( MessageAdapterFactory.CONNECTION_PASSWD );
		sendName = (String)env.get( MessageAdapterFactory.DEST_SEND );
		recvName = (String)env.get( MessageAdapterFactory.DEST_RECV );

		connect( qmgr, host, port, channel, user, passwd );
	}

	/**
	* MQに接続します。
	* @param	qmgr	キューマネージャ名
	* @param	host	ホスト名
	* @param	port	ポート番号
	* @param	channel	チャネル名
	* @param	user	ユーザ名
	* @param	passwd	パスワード
	*/
	public void connect( String qmgr, String host, String port, String channel,
			String user, String passwd )
		throws Exception
	{
		MQQueueConnectionFactory conFactory = new MQQueueConnectionFactory();
		if ( qmgr == null ) {
			qmgr = "";
		}
		CmnLog.debug( this, "connect QMGR:" + qmgr + " channel:" + channel
							+ " host=" + host + " port:" + port
							+ " send=" + sendName + " recv=" + recvName );

		if ( host != null && port != null && channel != null ) {
			MQEnvironment.hostname  = host;
			MQEnvironment.port      = Integer.parseInt( port );
			MQEnvironment.channel   = channel;
			if ( user != null ) {
				MQEnvironment.userID    = user;
			}
			if ( passwd != null ) {
				MQEnvironment.password  = passwd;
			}
			MQEnvironment.properties.put( MQC.TRANSPORT_PROPERTY,
											MQC.TRANSPORT_MQSERIES );
			conFactory.setHostName( MQEnvironment.hostname );
			conFactory.setPort( MQEnvironment.port );
			conFactory.setChannel( MQEnvironment.channel );
			conFactory.setQueueManager( qmgr );
			conFactory.setTransportType( JMSC.MQJMS_TP_CLIENT_MQ_TCPIP );
		}
		conFactory.setQueueManager( qmgr );

		try {
			if ( user != null ) {
				connection = conFactory.createQueueConnection( user, passwd );
			} else {
				connection = conFactory.createQueueConnection();
			}
			connection.setExceptionListener( this );

			sndSession = connection.createSession( false,
							Session.AUTO_ACKNOWLEDGE);
			rcvSession = connection.createSession( false,
							Session.AUTO_ACKNOWLEDGE);
		} catch ( JMSException jex ) {
			Exception lex = jex.getLinkedException();
			if ( lex != null ) {
				CmnLog.error( this, "linked exception:" + lex );
			}
			throw( jex );
		}
	}

	/**
	*
	*/
	public void createSender( String queue, boolean use ) throws Exception {
		super.createSender( queue, false );
	}

	/**
	*
	*/
	public void createReceiver( String queue, String cond, boolean wait, boolean use)
		throws Exception
	{
		super.createReceiver( queue, cond, wait, false );
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		config.addChild( MQ_QMGR, qmgr );
		config.addChild( MQ_HOST, host );
		config.addChild( MQ_PORT, port );
		config.addChild( MQ_CHANNEL, channel );
		config.addChild( MessageAdapterFactory.CONNECTION_USER, user );
		config.addChild( MessageAdapterFactory.CONNECTION_PASSWD, passwd );
		if ( sendName != null ) {
			config.addChild( MessageAdapterFactory.DEST_SEND, sendName );
		}
		if ( recvName != null ) {
			config.addChild( MessageAdapterFactory.DEST_RECV, recvName );
		}
	}

}
