package	jp.co.sds_corp.util.msg.message.jms;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.jms.BytesMessage;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.TextMessage;

import jp.co.sds_corp.util.msg.IMessage;

/**
* 共通メッセージクラス
*/
public class JmsMessage implements IMessage
{
	Type	type;
	Message	message;
	Object	object;

	public JmsMessage( Type type, Session session ) throws Exception {
		setType( type );
		switch ( type ) {
		case TEXT: message = session.createTextMessage(); break;
		case BYTE: message = session.createBytesMessage(); break;
		case MAP:
			message = session.createMapMessage();
			object = new HashMap<String,Object>();
			break;
		case OBJECT: message = session.createObjectMessage(); break;
		default:
			throw( new RuntimeException( "type[" + type + "] not supported" ));
		}
	}

	public JmsMessage( Message msg ) throws Exception {
		setMessage( msg );
	}

	public Type getType() {
		return this.type;
	}
	public void setType( Type type ) {
		this.type = type;
	}

	public Object getObject() throws Exception {
		return object;
	}
	public void setObject( Object object ) throws Exception {
		this.object = object;
		switch ( type ) {
		case TEXT:
			TextMessage textMsg = (TextMessage )message;
			textMsg.setText( (String )object );
			break;
		case BYTE:
			BytesMessage byteMsg = (BytesMessage )message;
			byteMsg.writeUTF( (String )object );
			break;
		case MAP:
			MapMessage mapMsg = (MapMessage )message;
			Map<String,Object> map = (Map<String,Object> )object;
			Iterator itr = map.keySet().iterator();
			while ( itr.hasNext() ) {
				String key = (String )itr.next();
				mapMsg.setObject( key, map.get( key ));
			}
			break;
		case OBJECT:
			ObjectMessage objMsg = (ObjectMessage )message;
			objMsg.setObject( (Serializable )object );
			break;
		}
	}

	public Object getObject( String key ) throws Exception {
		MapMessage mapMsg = (MapMessage )message;
		Object ret = mapMsg.getObject( key );
		return ret;
	}
	public void setObject( String key, Object obj ) throws Exception {
		Map<String,Object> map = (Map<String,Object> )object;
		map.put( key, obj );
		MapMessage mapMsg = (MapMessage )message;
		mapMsg.setObject( key, obj );
	}

	public Message getMessage() throws Exception {
		return message;
	}
	public void setMessage( Message message ) throws Exception {
		Type type;
		if ( message instanceof TextMessage ) {
			type = IMessage.Type.TEXT;
			TextMessage textMsg = (TextMessage )message;
			object = textMsg.getText();
		} else
		if ( message instanceof BytesMessage ) {
			type = IMessage.Type.BYTE;
			BytesMessage byteMsg = (BytesMessage )message;
			object = byteMsg.readUTF();
		} else
		if ( message instanceof MapMessage ) {
			type = IMessage.Type.MAP;
			MapMessage mapMsg = (MapMessage )message;
			Map<String,Object>map = new HashMap<String,Object>();
			Enumeration en = mapMsg.getMapNames();
			while ( en.hasMoreElements()) {
				String key = (String )en.nextElement();
				Object obj = mapMsg.getObject( key );
				map.put( key, obj );
			}
			object = map;
		} else
		if ( message instanceof ObjectMessage ) {
			type = IMessage.Type.OBJECT;
			ObjectMessage objMsg = (ObjectMessage )message;
			object = objMsg.getObject();
		} else {
			throw( new Exception( "Unknown message class:"
					+ "[" + message.getClass().getName() + "]" ));
		}
		setType( type );
		this.message = message;
	}

	/**
	* メッセージの複製を作成します
	* @param	msg		コピー元のメッセージ
	*/
	public void copyMessage( IMessage msg ) throws Exception {
		if ( getType() != msg.getType()) {
			throw( new Exception( "type mismatch." ));
		}
		if ( msg instanceof JmsMessage ) {
			setObject( ((JmsMessage )msg).getObject() );
			Message orgmsg = ((JmsMessage )msg).getMessage();
			copyProperties( orgmsg );
		} else {
			throw( new Exception( "Illegal message class:"
					+ "[" + msg.getClass().getName() + "]" ));
		}
	}

	protected void copyProperties( Message msg ) throws Exception {
		message.setJMSReplyTo( msg.getJMSReplyTo());
		message.setJMSTimestamp( msg.getJMSTimestamp() );
		message.setJMSMessageID( msg.getJMSMessageID() );
		message.setJMSCorrelationID( msg.getJMSCorrelationID() );

		if ( msg.propertyExists( "Sender" )) {
			message.setStringProperty( "Sender", msg.getStringProperty( "Sender" ));
		}
		if ( msg.propertyExists( "Address" )) {
			message.setStringProperty( "Address", msg.getStringProperty("Address"));
		}
		if ( msg.propertyExists( "Origin" )) {
			message.setStringProperty( "Origin", msg.getStringProperty( "Origin" ));
		}
		if ( msg.propertyExists( "Text" )) {
			message.setStringProperty( "Text", msg.getStringProperty( "Text" ));
		}
		if ( msg.propertyExists( "Subject" )) {
			message.setStringProperty( "Subject", msg.getStringProperty( "Subject" ));
		}
		if ( msg.propertyExists( "Primarykey" )) {
			message.setStringProperty( "Primarykey", msg.getStringProperty( "Primarykey" ));
		}
	} 

	public String toString() {
		String ret = getClass().getName()
			+ "[" + type + "]" + object
			+ ":" + message;
		return ret;
	}

}
